﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Maps.MapControl.WPF;
using Microsoft.Maps.MapControl.WPF.Core;
using Microsoft.Maps.MapControl.WPF;
using TrailMap.TileSource;
using TrailMap.Controls;
using TrailMap.Utils;
using GpxToolsLib;
using TrailMap.Projection;
using System.Threading;

namespace TrailMap
{
    public partial class MainPage : UserControl
    {
        List<OverlayItem> overlays = new List<OverlayItem>();

        private readonly Color highlighColor = Colors.Blue;

        ComboBox mapSource = new ComboBox();
        private RadioButton bttnNormal = new RadioButton();
        private RadioButton bttnSatellite = new RadioButton();
        private RadioButton bttnHybrid = new RadioButton();
        private RadioButton bttnTerrain = new RadioButton();
        MapLayer tileLayer = new MapLayer();
        MapLayer drawLayer = new MapLayer();
        ContextMenu pushPinMenu = new ContextMenu();

        TileSourceProvider sourceProvider = new TileSourceProvider();

        MapType mapType = MapType.Normal;
        IMapProvider currentSource;

        private MapTool m_currentTool = MapTool.View;

        #region ctor

        void MapForeground_TemplateApplied(object sender, EventArgs e)
        {
            //map.MapForeground.NavigationBar.TemplateApplied += NavigationBar_TemplateApplied;
        }

        void NavigationBar_TemplateApplied(object sender, EventArgs e)
        {
            //map.MapForeground.NavigationBar.HorizontalPanel.Children.Clear();

            

            //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(mapSource);
            //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnNormal);
            //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnSatellite);
            //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnHybrid);
        }

        void drawMap_OutputTrackChanged(bool doNotChangeView)
        {
            DrawTrack(doNotChangeView);
        }

        public MainPage()
        {
            InitializeComponent();

            lastTrackSegment.Stroke = new SolidColorBrush(highlighColor);
            lastTrackSegment.StrokeDashArray = new DoubleCollection{1.0, 1.0};
            lastTrackSegment.StrokeThickness = 5;

            bttnNormal.Content = "Normal";
            bttnNormal.Click += bttnNormal_Click;
            bttnNormal.IsChecked = true;
            bttnSatellite.Content = "Satellite";
            bttnSatellite.Click += bttnSatellite_Click;
            bttnHybrid.Content = "Hybrid";
            bttnHybrid.Click += bttnHybrid_Click;
            bttnTerrain.Content = "Terrain";
            bttnTerrain.Click += bttnTerrain_Click;

            mapSource.Items.Add("Bing");
            mapSource.Items.Add("Google");
            mapSource.Items.Add("Yahoo");
            mapSource.Items.Add("CloudMade");
            mapSource.Items.Add("OpenStreet");
            mapSource.Items.Add("BingAsOverlay");
            mapSource.SelectedIndex = 0;

            comboOverlay.Items.Add("Google");
            comboOverlay.Items.Add("Yahoo");
            comboOverlay.Items.Add("CloudMade");
            comboOverlay.Items.Add("OpenStreet");
            comboOverlay.Items.Add("BingAsOverlay");
            comboOverlay.SelectedIndex = -1;

            comboToolMode.SelectedIndex = 0;

            listOfOverlays.ItemsSource = null;

            mapSource.SelectionChanged += mapSource_SelectionChanged;

            //map.MapForeground.TemplateApplied += MapForeground_TemplateApplied;
            map.MouseLeftButtonDown += new MouseButtonEventHandler(map_MouseLeftButtonDown);
            map.MouseMove += new MouseEventHandler(map_MouseMove);
            map.MouseLeftButtonUp += new MouseButtonEventHandler(map_MouseLeftButtonUp);
            drawMap.OutputTrackChanged += drawMap_OutputTrackChanged;
            drawMap.InitializeDrawTrack(this);
            drawTool.InitializeDrawTool(this);

           // WebRequest.RegisterPrefix("http://", WebRequestCreator.ClientHttp);
        }

        #endregion

        #region Properties
        public IMapProvider CurrentSource
        {
            get
            {
                if (currentSource == null)
                {
                    IMapProvider retVal = this.SourceProvider.BingTileSource;
                    retVal.MapMode = mapType;
                    return retVal;
                }
                return currentSource;
            }
        }
        public MapTool CurrentTool
        {
            get { return m_currentTool; }
        }

        public TileSourceProvider SourceProvider
        {
            get { return sourceProvider; }
        }
        public DrawTool DrawTool
        {
            get { return drawTool; }
        }
        public DrawTrack DrawTrackCtrl
        {
            get { return drawMap; }
        }
        #endregion
        
        #region Mouse Handling
        void map_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (this.CurrentTool == MapTool.CaptureTiles &&
                customMapArea.Action == CustomAction.Area)
            {
                customMapArea.AreaData.Flag = false;
                customMapArea.AreaData.EtLocation = map.ViewportPointToLocation(e.GetPosition(map));

                DrawRect(customMapArea.AreaData.StLocation, customMapArea.AreaData.EtLocation);
            }
        }

        void map_MouseMove(object sender, MouseEventArgs e)
        {
            if (this.CurrentTool == MapTool.CaptureTiles &&
                customMapArea.Action == CustomAction.Area &&
                customMapArea.AreaData.Flag)
            {
                customMapArea.AreaData.EtLocation = map.ViewportPointToLocation(e.GetPosition(map));

                DrawRect(customMapArea.AreaData.StLocation, customMapArea.AreaData.EtLocation);
            }
            else if (this.CurrentTool == MapTool.DrawTrail && drawMap.CurrentEditTrack != null)
            {
                 ClearLastSegment();

                if (drawTool.CurrentSelection == DrawToolSelection.DrawTrackAtEnd)
                {
                    if (drawMap.CurrentEditTrack != null && drawMap.CurrentEditTrack.PolyLine.Locations.Count > 0)
                    {

                        lastTrackSegment.Locations = new LocationCollection { drawMap.CurrentEditTrack.PolyLine.Locations[drawMap.CurrentEditTrack.PolyLine.Locations.Count - 1],
                        map.ViewportPointToLocation(e.GetPosition(map))};

                        drawLayer.Children.Add(lastTrackSegment);
                    }
                }
                else if (drawTool.CurrentSelection == DrawToolSelection.DrawTrackAtBegin)
                {
                    if (drawMap.CurrentEditTrack != null && drawMap.CurrentEditTrack.PolyLine.Locations.Count > 0)
                    {

                        lastTrackSegment.Locations = new LocationCollection { drawMap.CurrentEditTrack.PolyLine.Locations[0],
                        map.ViewportPointToLocation(e.GetPosition(map))};

                        drawLayer.Children.Add(lastTrackSegment);
                    }
                }
            }
            else if (this.CurrentTool == MapTool.DrawTrail && drawTool.CurrentSelection == DrawToolSelection.Waypoint)
            {

                lastWaypoint.Location = map.ViewportPointToLocation(e.GetPosition(map));

                if (!drawLayer.Children.Contains(lastWaypoint))
                {
                    drawLayer.Children.Add(lastWaypoint);
                }
                
            }
            else if (this.CurrentTool == MapTool.DrawTrail && drawTool.CurrentSelection == DrawToolSelection.Hand)
            {
                if (drawLayer.Children.Contains(lastWaypoint))
                {
                    drawLayer.Children.Remove(lastWaypoint);
                }
            }

        }

        private void ClearLastSegment()
        {
            if (drawLayer.Children.Contains(lastTrackSegment))
            {
                drawLayer.Children.Remove(lastTrackSegment);
            }
        }


        MapPolyline lastTrackSegment = new MapPolyline();
        Pushpin lastWaypoint = new Pushpin();

        void map_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (this.CurrentTool == MapTool.CaptureTiles &&
                customMapArea.Action == CustomAction.Area)
            {
                customMapArea.AreaData.StLocation = map.ViewportPointToLocation(e.GetPosition(map));
                customMapArea.AreaData.Flag = true;
            }
            else if (this.CurrentTool == MapTool.DrawTrail && drawMap.CurrentEditTrack != null)
            {
                ClearLastSegment();

                if (drawTool.CurrentSelection == DrawToolSelection.DrawTrackAtEnd)
                {
                    if (drawMap.CurrentEditTrack != null)
                    {
                        Location loc =  map.ViewportPointToLocation(e.GetPosition(map));
                        drawMap.CurrentEditTrack.Track.trkseg.Add(new trkpt { lat = loc.Latitude.ToString(), lon = loc.Longitude.ToString(), ele = loc.Altitude.ToString(), time = DateTime.Now });
                        DrawTrack(true);
                    }
                }
                else if (drawTool.CurrentSelection == DrawToolSelection.DrawTrackAtBegin)
                {
                    if (drawMap.CurrentEditTrack != null)
                    {
                        Location loc = map.ViewportPointToLocation(e.GetPosition(map));
                        drawMap.CurrentEditTrack.Track.trkseg.Insert(0,new trkpt { lat = loc.Latitude.ToString(), lon = loc.Longitude.ToString(), ele = loc.Altitude.ToString(), time = DateTime.Now });
                        DrawTrack(true);
                    }
                }
            }
            else if (this.CurrentTool == MapTool.DrawTrail && drawTool.CurrentSelection == DrawToolSelection.Waypoint)
            {
                drawLayer.Children.Remove(lastWaypoint);
                drawMap.AddWayPoints(map.ViewportPointToLocation(e.GetPosition(map)));
                DrawTrack(true);
            }
        } 
        #endregion

        #region MapType Buttons Handler
        void bttnTerrain_Click(object sender, RoutedEventArgs e)
        {
            mapType = MapType.Terrain;
            UpdateMapType();
        }

        void bttnHybrid_Click(object sender, RoutedEventArgs e)
        {
            mapType = MapType.Hybrid;
            UpdateMapType();
        }

        void bttnSatellite_Click(object sender, RoutedEventArgs e)
        {
            mapType = MapType.Satellite;
            UpdateMapType();
        }

        void bttnNormal_Click(object sender, RoutedEventArgs e)
        {
            mapType = MapType.Normal;
            UpdateMapType();
        } 
        #endregion

        #region Map Source and Tool Selection Handling
        void mapSource_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            MaporOverlayChanged();
        }
        private void comboToolMode_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {

            m_currentTool = (MapTool)comboToolMode.SelectedIndex;

            UpdateTool();
        } 
        #endregion

        #region Utility Methods
        

        private void UpdateMapType()
        {
            bttnNormal.IsChecked = false;
            bttnSatellite.IsChecked = false;
            bttnHybrid.IsChecked = false;
            bttnTerrain.IsChecked = false;

            map.Children.Clear();

            map.Children.Add(tileLayer);

            tileLayer.Visibility = System.Windows.Visibility.Collapsed;

            if (currentSource != null)
            {
                tileLayer.TileSources.Clear();
            }

            #region Main Map Source Management
            switch (mapType)
            {
                case MapType.Normal:
                    bttnNormal.IsChecked = true;
                    if (currentSource != null)
                    {
                        currentSource.MapMode = mapType;
                    }
                    else
                    {
                        map.Mode = new RoadMode();
                    }
                    break;
                case MapType.Hybrid:
                    bttnHybrid.IsChecked = true;
                    if (currentSource != null)
                    {
                        currentSource.MapMode = mapType;
                        tileLayer.TileSources.Add(currentSource.HybridBase);
                    }
                    else
                    {
                        AerialMode aerialMode = new AerialMode();
                        aerialMode.Labels = true;
                        map.Mode = aerialMode;
                    }
                    break;
                case MapType.Satellite:
                    bttnSatellite.IsChecked = true;
                    if (currentSource != null)
                    {
                        currentSource.MapMode = mapType;
                    }
                    else
                    {
                        AerialMode aerialMode = new AerialMode();
                        aerialMode.Labels = false;
                        map.Mode = aerialMode;
                    }
                    break;
                case MapType.Terrain:
                    bttnTerrain.IsChecked = true;
                    if (currentSource != null)
                    {
                        currentSource.MapMode = mapType;
                    }
                    else
                    {
                        map.Mode = new RoadMode();
                    }
                    break;
            }
            #endregion

            if (currentSource != null)
            {
                tileLayer.TileSources.Add(currentSource as Microsoft.Maps.MapControl.WPF.TileSource);
                tileLayer.Visibility = System.Windows.Visibility.Visible;
            }

            foreach (OverlayItem item in overlays)
            {
                MapLayer layer = item.Provider;
                map.Children.Add(layer);
                layer.Visibility = System.Windows.Visibility.Visible;
            }
            if (this.CurrentTool == MapTool.DrawTrail)
            {
                map.Children.Add(drawLayer);
            }
        }

        private void DrawTrack(bool doNotChangeView)
        {
            drawLayer.Children.Clear();

            LocationCollection locations = null;

            List<TrackPoints> trkPoints = drawMap.OutputTrks;
            TrackPoints currentTrackPoint = null;

            foreach (TrackPoints item in trkPoints)
            {
                drawLayer.Children.Add(item.PolyLine);

                if (drawMap.CurrentEditTrack != null)
                {
                    if (item.PushPins.Count > 0)
                    {
                        foreach (Pushpin pushpinItem in item.PushPins)
                        {
                            drawLayer.Children.Add(pushpinItem);
                        }

                        locations = item.PolyLine.Locations;
                        currentTrackPoint = item;
                    }
                }
                
            }

            List<WaypointPin> wayPoints = drawMap.CurrentWayPoints;

            if (drawMap.CurrentEditTrack == null && wayPoints != null)
            {
                foreach (WaypointPin pushpinItem in wayPoints)
                {
                    drawLayer.Children.Add(pushpinItem);
                }
            }

            //foreach

            if (doNotChangeView == false)
            {

                if (locations == null && trkPoints.Count > 0)
                {
                    locations = trkPoints[0].PolyLine.Locations;
                    currentTrackPoint = trkPoints[0];
                }

                if (locations != null)
                {
                    map.SetView(new LocationRect(locations));
                }
                else if(wayPoints != null && wayPoints.Count > 0)
                {
                    map.SetView(new LocationRect(wayPoints[0].Location, wayPoints[wayPoints.Count - 1].Location));
                }

                //if (currentTrackPoint != null)
                //{
                //    elePlot.Track = currentTrackPoint;
                //}
            }
        }

        private void DrawRect(Location stLoc, Location etLoc)
        {
            map.Children.Remove(customMapArea.Polygon);

            LocationRect rect = new LocationRect(stLoc, etLoc);

            customMapArea.Polygon.Locations = new LocationCollection() { rect.Northwest, rect.Northeast, rect.Southeast, rect.Southwest };

            map.Children.Add(customMapArea.Polygon);
        }

        private void UpdateTool()
        {
            
            // TODO: Add event handler implementation here.
            customMapArea.Visibility = System.Windows.Visibility.Collapsed;
            drawMap.Visibility = System.Windows.Visibility.Collapsed;
            elePlot.Visibility = System.Windows.Visibility.Collapsed;
            drawTool.Visibility = System.Windows.Visibility.Collapsed;
            BusyWindow.Visibility = System.Windows.Visibility.Visible;

            if (map.Children.Contains(drawLayer))
            {
                map.Children.Remove(drawLayer);
            }

            switch (this.CurrentTool)
            {
                case MapTool.CaptureTiles:
                    UpdateMapType();
                    customMapArea.SetMap(map, this);
                    customMapArea.Visibility = System.Windows.Visibility.Visible;
                    break;
                case MapTool.DrawTrail:
                    drawMap.Visibility = System.Windows.Visibility.Visible;
                    drawTool.Visibility = System.Windows.Visibility.Visible;
                    //elePlot.Visibility = System.Windows.Visibility.Visible;
                    UpdateMapType();
                    if (!map.Children.Contains(drawLayer))
                    {
                        map.Children.Add(drawLayer);
                    }
                    DrawTrack(false);
                    break;
            }
        } 
        #endregion

        #region Overlay Management
        private void MaporOverlayChanged()
        {
            //map.MapForeground.NavigationBar.HorizontalPanel.Children.Remove(bttnNormal);
            //map.MapForeground.NavigationBar.HorizontalPanel.Children.Remove(bttnSatellite);
            //map.MapForeground.NavigationBar.HorizontalPanel.Children.Remove(bttnHybrid);
            //map.MapForeground.NavigationBar.HorizontalPanel.Children.Remove(bttnTerrain);
            tileLayer.Visibility = System.Windows.Visibility.Collapsed;
            currentSource = null;

            switch (mapSource.SelectedIndex)
            {
                case 1:
                    currentSource = this.SourceProvider.GoogleSource;
                    map.Mode = new MercatorMode();

                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnNormal);
                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnSatellite);
                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnHybrid);
                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnTerrain);
                    break;
                case 2:
                    currentSource = this.SourceProvider.YahooSource;
                    map.Mode = new MercatorMode();

                    if (mapType == MapType.Terrain)
                    {
                        mapType = MapType.Normal;
                    }

                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnNormal);
                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnSatellite);
                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnHybrid);
                    break;
                case 3:
                    currentSource = this.SourceProvider.CmSource;
                    map.Mode = new MercatorMode();

                    if (mapType == MapType.Satellite || mapType == MapType.Hybrid)
                    {
                        mapType = MapType.Normal;
                    }

                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnNormal);
                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnTerrain);
                    break;
                case 4:
                    currentSource = this.SourceProvider.OpenStreetMapTileSource;
                    map.Mode = new MercatorMode();

                    if (mapType == MapType.Satellite || mapType == MapType.Hybrid || mapType == MapType.Terrain)
                    {
                        mapType = MapType.Normal;
                    }

                    map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnNormal);
                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnSatellite);
                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnHybrid);
                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnTerrain);
                    break;
                case 5:
                    currentSource = this.SourceProvider.BingTileSource;
                    map.Mode = new MercatorMode();
                    if (mapType == MapType.Terrain)
                    {
                        mapType = MapType.Normal;
                    }

                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnNormal);
                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnSatellite);
                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnHybrid);
                    break;
                default:
                    tileLayer.Visibility = System.Windows.Visibility.Collapsed;
                    tileLayer.TileSources.Clear();
                    if (mapType == MapType.Terrain)
                    {
                        mapType = MapType.Normal;
                    }

                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnNormal);
                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnSatellite);
                    //map.MapForeground.NavigationBar.HorizontalPanel.Children.Add(bttnHybrid);
                    break;
            }

            UpdateMapType();
        }



        private void bttnAdd_Click(object sender, RoutedEventArgs e)
        {
            string selectedItem = comboOverlay.SelectedItem as string;
            if (!string.IsNullOrWhiteSpace(selectedItem))
            {
                var query = from item in overlays where item.DisplayName == selectedItem select item;

                if (query.Count() == 0)
                {
                    overlays.Add(new OverlayItem(selectedItem, 50, comboOverlay.SelectedIndex + 1, MapType.Normal));
                    listOfOverlays.ItemsSource = null;
                    listOfOverlays.ItemsSource = overlays;
                    UpdateMapType();
                }
            }
        }
        public string[] Buttons
        {
            get
            {
                return new string[] { "N", "S", "H", "T" };
            }
        }
        public List<OverlayItem> Overlays
        {
            get { return overlays; }
        }

        private void bttnRemove_Click(object sender, RoutedEventArgs e)
        {
            OverlayItem selectedItem = listOfOverlays.SelectedItem as OverlayItem;
            if (selectedItem != null)
            {
                overlays.Remove(selectedItem);
                listOfOverlays.ItemsSource = null;
                listOfOverlays.ItemsSource = overlays;
                UpdateMapType();
            }
        } 
        #endregion
       
    }
}
